==> fastrack-property-v1.1.18-254634.tar: src/Offer.php <==
<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\HiddenField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDate;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\Subsites\Model\Subsite;
use SilverStripe\Subsites\State\SubsiteState;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use Symbiote\GridFieldExtensions\GridFieldEditableColumns;

/**
 * @property string Title
 * @property string Name
 * @property string Excerpt
 * @property string Content
 * @property string Slug
 * @property string Price
 * @property string Url
 * @property DBDate Start
 * @property DBDate End
 * @property string Terms
 * @property bool Active
 * @property bool Featured
 * @property int SortOrder
 * @method ManyManyList Properties()
 * @method Imgix Image()
 */
class Offer extends DataObject implements PermissionProvider
{
	private static string $table_name = 'offer';

	private static array $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Slug' => 'Varchar(255)',
		'Price' => 'Varchar(255)',
		'Url' => 'Varchar(255)',
		'Start' => 'Date',
		'End' => 'Date',
		'Terms' => 'HTMLText',
		'Active' => 'Boolean',
		'Featured' => 'Boolean',
		'SortOrder' => 'Int',
	];

	private static array $has_one = [
		'Subsite' => Subsite::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,

	];

	private static array $many_many = [
		'Properties' => Property::class,
	];

	private static array $many_many_extraFields = [
		'Properties' => [
			'OfferSortOrder' => 'Int',
			'Url' => 'Varchar',
		],
	];

	private static string $default_sort = 'SortOrder';

	private static array $indexes = [
		'Slug' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
		'Terms',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		if (class_exists(Subsite::class)){
			$fields->push(new HiddenField('SubsiteID','SubsiteID', SubsiteState::singleton()->getSubsiteId()));
		}
		$gridConfiguration = GridFieldConfig_RelationEditor::create(100);
		$gridConfiguration->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfiguration->removeComponentsByType(GridFieldEditButton::class);
		$gridConfiguration->addComponent(GridFieldAddExistingSearchButton::create());
		$gridConfiguration->addComponent(GridFieldEditableColumns::create());
		$grid = new GridField('Properties', 'Properties', $this->Properties(), $gridConfiguration);

		$grid->getConfig()->getComponentByType(GridFieldEditableColumns::class)->setDisplayFields([
			'Url' => [
				'title' => 'URL',
				'field' => TextField::class,
			]
		]);



		$fields->addFieldToTab('Root.Properties', $grid);
		ModelUtilities::hideFields($this, $fields);
		ModelUtilities::hideConfigFields($this, $fields);
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		return $fields;
	}

	public function onBeforeWrite() {
    	$this->Title = $this->Name;
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	public function isValid(): bool
	{
		$today = strtotime(date('Y-m-d'));
		if (!$this->Active) {
			return false;
		}
		if ($this->End && ($today > strtotime($this->End))) {
			return false;
		}
		if ($this->Start && ($today < strtotime($this->Start))) {
			return false;
		}
		$properties = $this->Properties()->filterByCallback(function (Property $property) {
			return $property->isPublished() && $property->isActive();
		})->count();
		if ($properties === 0) {
			return false;
		}
		return true;
	}

	public static function getValidOffers(): ArrayList
	{
		return Offer::get()->filterByCallback(function (Offer $offer) {
			return $offer->isValid();
		});
	}

	public function Link(): string
	{
		$link = ArrayList::create();
		$property = $this->Properties()->first();
		if ($property) {
			$link->push($property->OffersLink());
		}
		$link->push($this->Slug);
		$this->extend('updateOfferLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function CMSEditLink(): string
	{
		$admin = Injector::inst()->get(PropertyAdmin::class);
		return Controller::join_links([
			$admin->Link('offer'),
			'EditForm',
			'field',
			'offer',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink(): string
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions()
	{
		$permissions = [
			'MANAGE_OFFERS' => [
				'name' => 'Manage offers',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->isValid();
			}
		}
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

}

==> fastrack-property-v1.1.19-288437.tar: src/Offer.php <==
<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Injector\Injector;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldEditButton;
use SilverStripe\Forms\HiddenField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\FieldType\DBDate;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;
use SilverStripe\Subsites\Model\Subsite;
use SilverStripe\Subsites\State\SubsiteState;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use Symbiote\GridFieldExtensions\GridFieldEditableColumns;

/**
 * @property string Title
 * @property string Name
 * @property string Excerpt
 * @property string Content
 * @property string Slug
 * @property string Price
 * @property string Url
 * @property DBDate Start
 * @property DBDate End
 * @property string Terms
 * @property bool Active
 * @property bool Featured
 * @property int SortOrder
 * @method ManyManyList Properties()
 * @method Imgix Image()
 */
class Offer extends DataObject implements PermissionProvider
{
	private static string $table_name = 'offer';

	private static array $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Slug' => 'Varchar(255)',
		'Price' => 'Varchar(255)',
		'Url' => 'Varchar(255)',
		'Start' => 'Date',
		'End' => 'Date',
		'Terms' => 'HTMLText',
		'Active' => 'Boolean',
		'Featured' => 'Boolean',
		'SortOrder' => 'Int',
	];

	private static array $has_one = [
		'Subsite' => Subsite::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,

	];

	private static array $many_many = [
		'Properties' => Property::class,
	];

	private static array $many_many_extraFields = [
		'Properties' => [
			'OfferSortOrder' => 'Int',
			'Url' => 'Varchar',
		],
	];

	private static string $default_sort = 'SortOrder';

	private static array $indexes = [
		'Slug' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
		'Terms',
	];

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		if (class_exists(Subsite::class)){
			$fields->push(new HiddenField('SubsiteID','SubsiteID', SubsiteState::singleton()->getSubsiteId()));
		}
		$gridConfiguration = GridFieldConfig_RelationEditor::create(100);
		$gridConfiguration->removeComponentsByType(GridFieldAddNewButton::class);
		$gridConfiguration->removeComponentsByType(GridFieldEditButton::class);
		$gridConfiguration->addComponent(new GridFieldAddExistingSearchButton());
		$gridConfiguration->addComponent(new GridFieldEditableColumns());
		$grid = new GridField('Properties', 'Properties', $this->Properties(), $gridConfiguration);

		$grid->getConfig()->getComponentByType(GridFieldEditableColumns::class)->setDisplayFields([
			'Url' => [
				'title' => 'URL',
				'field' => TextField::class,
			]
		]);



		$fields->addFieldToTab('Root.Properties', $grid);
		ModelUtilities::hideFields($this, $fields);
		ModelUtilities::hideConfigFields($this, $fields);
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		return $fields;
	}

	public function onBeforeWrite() {
    	$this->Title = $this->Name;
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	public function isValid(): bool
	{
		$today = strtotime(date('Y-m-d'));
		if (!$this->Active) {
			return false;
		}
		if ($this->End && ($today > strtotime($this->End))) {
			return false;
		}
		if ($this->Start && ($today < strtotime($this->Start))) {
			return false;
		}
		$properties = $this->Properties()->filterByCallback(function (Property $property) {
			return $property->isPublished() && $property->isActive();
		})->count();
		if ($properties === 0) {
			return false;
		}
		return true;
	}

	public static function getValidOffers(): ArrayList
	{
		return Offer::get()->filterByCallback(function (Offer $offer) {
			return $offer->isValid();
		});
	}

	public function Link(): string
	{
		$link = ArrayList::create();
		$property = $this->Properties()->first();
		if ($property) {
			$link->push($property->OffersLink());
		}
		$link->push($this->Slug);
		$this->extend('updateOfferLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function CMSEditLink(): string
	{
		$admin = Injector::inst()->get(PropertyAdmin::class);
		return Controller::join_links([
			$admin->Link('offer'),
			'EditForm',
			'field',
			'offer',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink(): string
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions()
	{
		$permissions = [
			'MANAGE_OFFERS' => [
				'name' => 'Manage offers',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->isValid();
			}
		}
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_OFFERS']);
	}

}
